package
{
	import actors.SimpleIsoActor;
	
	import as3isolib.display.IsoSprite;
	import as3isolib.display.IsoView;
	import as3isolib.display.primitive.IsoBox;
	import as3isolib.display.scene.IsoGrid;
	import as3isolib.display.scene.IsoScene;
	import as3isolib.geom.Pt;
	import as3isolib.graphics.Stroke;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.geom.Point;
	import flash.ui.Keyboard;
	
	import objects.IsoSilla;
	import objects.IsoSimpleAsset;

	
	public class SimpleIsoCharacterDemo extends Sprite
	{
		private var viewPort:IsoView;
		private var scene:IsoScene;
		private var character:SimpleIsoActor;
		
		private const GRID_W:uint =8;
		private const GRID_L:uint =8;
		private const CELL_SIZE:uint= 35;
		
		private var obstacles:Vector.<IsoSprite>;
		
		public function SimpleIsoCharacterDemo()
		{
			setIso();
			
			addObstacles();
			
			addPerson();
			
		}
		
		private function addObstacles():void
		{
			obstacles = new Vector.<IsoSprite>();

			var arraypos:Vector.<Point> = new Vector.<Point>();
			
			arraypos.push(new Point(55,55));
			
			arraypos.push(new Point(200,200));
			
			arraypos.push(new Point(200,50));
			
			arraypos.push(new Point(120,220));
			
			arraypos.push(new Point(0,200));
			
			var num:uint = arraypos.length;
			for(var i:uint=0;i<num;i++)
			{
				var o:IsoSimpleAsset = new IsoSilla();
				
				o.moveTo(arraypos[i].x,arraypos[i].y,0);
				
				scene.addChild(o);
				obstacles.push(o);
			}
			
		}
		
		private function addPerson():void
		{
			//creamos personaje
			//la clase SimpleIsoActor extiende de IsoSprite
			character = new SimpleIsoActor();
			
			//posición inicial
			character.moveTo(140, 140, 0);
			
			//añadimos a la escena isométrica
			scene.addChild(character);
			
			//añadimos listeners de teclado
			stage.addEventListener(KeyboardEvent.KEY_DOWN,keydown_handler);
			stage.addEventListener(KeyboardEvent.KEY_UP,keyup_handler);
		}
		
		//variables que controlan 
		//si el usuario mantiene presionada una tecla
		private var left_key_down:Boolean=false;
		private var right_key_down:Boolean=false;
		private var up_key_down:Boolean=false;
		private var down_key_down:Boolean=false;
		
		//event listener que se encarga de actualizar las
		//variables de control de teclado (KEYDOWN)
		private function keydown_handler(e:KeyboardEvent):void
		{
			switch(e.keyCode)
			{
				case Keyboard.LEFT:
					left_key_down=true;
					break;
				
				case Keyboard.RIGHT:
					right_key_down=true;
					break;
				
				case Keyboard.UP:
					up_key_down=true;
					break;
				
				case Keyboard.DOWN:
					down_key_down=true;
					break;
			}
			
		}
		
		//event listener que se encarga de actualizar las
		//variables de control de teclado (KEYDOWN)
		private function keyup_handler(e:KeyboardEvent):void
		{
			switch(e.keyCode)
			{
				case Keyboard.LEFT:
					left_key_down=false;
					break;
				
				case Keyboard.RIGHT:
					right_key_down=false;
					break;
				
				case Keyboard.UP:
					up_key_down=false;
					break;
				
				case Keyboard.DOWN:
					down_key_down=false;
					break;
			}
		}
		
		private function setIso():void
		{
			
			//creamos vista y la configuramos
			viewPort = new IsoView();
			viewPort.showBorder=false;
			viewPort.setSize(539, 400);
			addChild(viewPort);
			
			//creamos escena 
			scene = new IsoScene();
			//y la añadimos a la vista
			viewPort.addScene(scene);
			
			//creamos un grid y lo añadimos a la escena
			//para referencia
			var grid:IsoGrid = new IsoGrid();
			grid.setGridSize(GRID_W,GRID_L,0);
			grid.cellSize = CELL_SIZE;
			grid.showOrigin=true;
			grid.stroke = new Stroke(1,0x999999,1);
			scene.addChild(grid);
			
			viewPort.centerOnPt(new Pt(0,100),false);
			
			//enterframe principal
			addEventListener(
				Event.ENTER_FRAME, 
				enterFrame_handler);	
		}
		
		private var moveX:int;
		private var moveY:int;
		private var sumXY:int;
		private var direction:String;
		private var do_move:Boolean;
		
		//MAIN LOOP
		private function enterFrame_handler(e:Event):void
		{
			sumXY=moveY=moveX=0;
			direction="";
			do_move=false;
			
			//conversión de teclas a traslaciones isométricas
			if(up_key_down)
			{
				moveX-=5;
				moveY-=5;	
			}
			
			if(down_key_down)
			{
				moveX+=5;
				moveY+=5;
			}
			
			if(left_key_down)
			{	
				moveX-=5;
				moveY+=5;
			}
			
			if(right_key_down)
			{
				moveX+=5;
				moveY-=5;
			}
			
			//ajustamos velocidad
			moveX = Math.max(Math.min(moveX,5),-5);
			moveY = Math.max(Math.min(moveY,5),-5);
			
			//comproblamos que no se produce colisión en el movimiento
			do_move = canMove();
			
			//"truco" para hacer un switch de 2 variables ;)
			sumXY = moveX*10 + moveY;
			switch(sumXY)
			{
				case 0:
					direction="";
					break;
				
				case 5://x:0,y:+5
					direction = "LeftFront";
					break;
				
				case -5://x:0,y:-5
					direction = "RightBack";
					break;
				case -55: //x:-5, y:-5
					direction = "Back";		
					break;
				
				case -50://x:-5, y:0
					direction = "LeftBack";
					break;
				
				case -45: //x:-5, y:+5
					direction = "Left";		
					break;
				
				case 55: //x:+5, y:+5
					direction = "Front";		
					break;
				
				case 50: //x:+5, y:-5
					direction = "RightFront";
					break;
				
				case 45: //x:+5, y:-5
					direction = "Right";		
					break;
				
			}
			
			if(do_move)
			{
				character.walkDirection(direction,moveX,moveY);
				
			}else
			{
				character.standDirection(direction);
			}
			
			//renderizamos escena
			scene.render();
		}
		
		//comprobar preventivamente si un movimiento de x,y pixels
		//provocaría una colisión con el obstáculo o
		private function checkIntersects(x:int,y:int,o:IsoSprite):Boolean
		{
			//muevo virtualmente el personaje (sin actualizar su posición gráficamente)
			character.moveBy(x,y,0);
			//compruebo si choca
			
			var intersects:Boolean = character.isoBounds.intersects(o.isoBounds);
			//lo devuelvo a su sitio
			character.moveBy(-x,-y,0);
			
			return intersects;
		}
		
		//comprobamos obstáculos
		private function canMove():Boolean
		{	
			//choco con algún obstáculo?
			var choca:Boolean;
			
			//loop de obstáculos en el stage
			for each(var o:IsoSprite in obstacles)
			{
				//si ya no me estoy moviendo, salgo del loop
				if( (moveX == 0) && (moveY == 0) )
					break;
				
				choca = checkIntersects(moveX,moveY,o);
				
				//si choca, voy a intentar "esquivar el obstáculo"
				if(choca)
				{
					//intento esquivarlo por el eje Y
					choca = checkIntersects(0,moveY,o);
					
					//si ya no choco, me quedo con este movimiento
					if(!choca)
					{
						moveX=0;
						break;
					}
					
					//intento esquivarlo por el eje X
					choca = checkIntersects(moveX,0,o);
					
					//si ya no choco, me quedo con este movimiento
					if(!choca)
					{
						moveY=0;
						break;
					}
					
					//si sigo aquí, es que me he quedado sin movimientos
					//no puedo seguir moviendo
					return false;
				}
				
			}
			
			//me podré mover si aún me queda algún moviemento
			//en el eje X o Y
			return ((moveX != 0) || (moveY != 0));
			
		}
		
		
	}//end class
}//end package